@@ -1103,189 +1103,17 @@ PVR_ERROR WaipuData::GetChannelGroupMembers(const kodi::addon::PVRChannelGroup&
   return PVR_ERROR_NO_ERROR;
 }
 
+
 PVR_ERROR WaipuData::GetEPGForChannel(int channelUid,
-                                      time_t start,
-                                      time_t end,
-                                      kodi::addon::PVREPGTagsResultSet& results)
+                                         time_t start,
+                                         time_t end,
+                                         kodi::addon::PVREPGTagsResultSet& results)
 {
   if (!IsConnected())
     return PVR_ERROR_SERVER_ERROR;
 
   LoadChannelData();
 
-  bool has_results = false;
-  bool epg_experimental = kodi::addon::GetSettingBoolean("epg_experimental");
-
-  for (const auto& channel : m_channels)
-  {
-    if (channel.iUniqueId != channelUid)
-      continue;
-
-    std::string startTime = Utils::TimeToString(start);
-    std::string endTime = Utils::TimeToString(end);
-
-    std::string jsonEpg = HttpGet("https://epg.waipu.tv/api/channels/" + channel.waipuID +
-                                  "/programs?startTime=" + std::string(startTime) +
-                                  "&stopTime=" + std::string(endTime));
-    kodi::Log(ADDON_LOG_DEBUG, "[epg-all] %s", jsonEpg.c_str());
-    if (jsonEpg.empty())
-    {
-      kodi::Log(ADDON_LOG_DEBUG, "%s - Empty server response", __FUNCTION__);
-      return PVR_ERROR_SERVER_ERROR;
-    }
-    jsonEpg = "{\"result\": " + jsonEpg + "}";
-
-    rapidjson::Document epgDoc;
-    epgDoc.Parse(jsonEpg.c_str());
-    if (epgDoc.HasParseError())
-    {
-      kodi::Log(ADDON_LOG_ERROR, "[%s] Error while parsing JSON", __FUNCTION__);
-      return PVR_ERROR_SERVER_ERROR;
-    }
-    kodi::Log(ADDON_LOG_DEBUG, "[epg] iterate entries");
-
-    kodi::Log(ADDON_LOG_DEBUG, "[epg] size: %i;", epgDoc["result"].Size());
-
-    for (const auto& epgData : epgDoc["result"].GetArray())
-    {
-      kodi::addon::PVREPGTag tag;
-
-      // generate a unique boadcast id
-      const std::string epg_bid = epgData["id"].GetString();
-      kodi::Log(ADDON_LOG_DEBUG, "[epg] epg_bid: %s;", epg_bid.c_str());
-      int dirtyID = Utils::GetIDDirty(epg_bid);
-      kodi::Log(ADDON_LOG_DEBUG, "[epg] epg_bid dirty: %i;", dirtyID);
-      tag.SetUniqueBroadcastId(dirtyID);
-
-      // channel ID
-      tag.SetUniqueChannelId(channel.iUniqueId);
-
-      /*// add streamUrlProvider if it is video on demand
-      if (myChannel.tvfuse && epgData.HasMember("streamUrlProvider") && !epgData["streamUrlProvider"].IsNull())
-      {
-        string streamUrlProvider = epgData["streamUrlProvider"].GetString();
-        kodi::Log(ADDON_LOG_DEBUG, "[epg] streamUrlProvider: %s;", streamUrlProvider.c_str());
-      }*/
-
-      unsigned int flags = EPG_TAG_FLAG_UNDEFINED;
-
-      // is recordable
-      bool isRecordable = !epgData["recordingForbidden"].GetBool();
-      kodi::Log(ADDON_LOG_DEBUG, "[epg] recordable: %i;", isRecordable);
-      if (isRecordable)
-      {
-        flags |= EPG_TAG_FLAG_IS_RECORDABLE_WAIPU;
-      }
-
-      // instantRestartAllowed
-      bool instantRestartAllowed = !epgData["instantRestartForbidden"].GetBool();
-      kodi::Log(ADDON_LOG_DEBUG, "[epg] instantRestartAllowed: %i;", instantRestartAllowed);
-      if (isRecordable)
-      {
-        flags |= EPG_TAG_FLAG_INSTANT_RESTART_ALLOWED_WAIPU;
-      }
-
-      // set title
-      tag.SetTitle(epgData["title"].GetString());
-      kodi::Log(ADDON_LOG_DEBUG, "[epg] title: %s;", epgData["title"].GetString());
-
-      // set startTime
-      const std::string entryStartTime = epgData["startTime"].GetString();
-      tag.SetStartTime(Utils::StringToTime(entryStartTime));
-
-      // set endTime
-      const std::string entryEndTime = epgData["stopTime"].GetString();
-      tag.SetEndTime(Utils::StringToTime(entryEndTime));
-
-      // tag.SetPlotOutline(myTag.strPlotOutline);
-
-      // set description
-      if (epgData.HasMember("description") && !epgData["description"].IsNull())
-      {
-        tag.SetPlot(epgData["description"].GetString());
-        kodi::Log(ADDON_LOG_DEBUG, "[epg] description: %s;", epgData["description"].GetString());
-      }
-
-      // epg preview image
-      if (m_epg_show_preview_images && epgData.HasMember("previewImages") &&
-          epgData["previewImages"].IsArray() && epgData["previewImages"].Size() > 0)
-      {
-        std::string tmp_img = epgData["previewImages"][0].GetString();
-        tmp_img += "?width=480&height=270";
-        tag.SetIconPath(tmp_img);
-        kodi::Log(ADDON_LOG_DEBUG, "[epg] previewImage: %s;", tmp_img.c_str());
-      }
-
-      // iSeriesNumber
-      if (epgData.HasMember("season") && !epgData["season"].IsNull())
-      {
-        tag.SetSeriesNumber(
-            Utils::StringToInt(epgData["season"].GetString(), EPG_TAG_INVALID_SERIES_EPISODE));
-        flags |= EPG_TAG_FLAG_IS_SERIES;
-      }
-      else
-      {
-        tag.SetSeriesNumber(EPG_TAG_INVALID_SERIES_EPISODE);
-      }
-      // episodeNumber
-      if (epgData.HasMember("episode") && epgData["episode"].IsString())
-      {
-        tag.SetEpisodeNumber(
-            Utils::StringToInt(epgData["episode"].GetString(), EPG_TAG_INVALID_SERIES_EPISODE));
-      }
-      else
-      {
-        tag.SetEpisodeNumber(EPG_TAG_INVALID_SERIES_EPISODE);
-      }
-
-      // episodeName
-      if (epgData.HasMember("episodeTitle") && !epgData["episodeTitle"].IsNull())
-      {
-        tag.SetEpisodeName(epgData["episodeTitle"].GetString());
-      }
-
-      // year
-      if (epgData.HasMember("year") && !epgData["year"].IsNull())
-      {
-        const int year = Utils::StringToInt(epgData["year"].GetString(), 1970);
-        if (year > 1970)
-          tag.SetYear(year);
-      }
-
-      // genre
-      if (epgData.HasMember("genreDisplayName") && !epgData["genreDisplayName"].IsNull())
-      {
-        const std::string genreStr = epgData["genreDisplayName"].GetString();
-        int genre = m_categories.Category(genreStr);
-        if (genre)
-        {
-          tag.SetGenreSubType(genre & 0x0F);
-          tag.SetGenreType(genre & 0xF0);
-        }
-        else
-        {
-          tag.SetGenreType(EPG_GENRE_USE_STRING);
-          tag.SetGenreSubType(0); /* not supported */
-          tag.SetGenreDescription(genreStr);
-        }
-      }
-      has_results = true;
-      tag.SetFlags(flags);
-      results.Add(tag);
-    }
-    if (epg_experimental && !has_results)
-    {
-      return GetEPGForChannelNew(channelUid, start, end, results);
-    }
-  }
-  return PVR_ERROR_NO_ERROR;
-}
-
-PVR_ERROR WaipuData::GetEPGForChannelNew(int channelUid,
-                                         time_t start,
-                                         time_t end,
-                                         kodi::addon::PVREPGTagsResultSet& results)
-{
   const int grid_align_hours = 4; // align 4h
 
   for (const auto& channel : m_channels)
@@ -1300,6 +1128,8 @@ PVR_ERROR WaipuData::GetEPGForChannelNew(int channelUid,
     kodi::Log(ADDON_LOG_DEBUG, "[epg-new] channel: %s", channelid.c_str());
 
     int limit = 32;
+    char startTimeBuf[30];
+    rapidjson::Document epgDoc;
 
     while (start < end)
     {
@@ -1309,7 +1139,7 @@ PVR_ERROR WaipuData::GetEPGForChannelNew(int channelUid,
       tm->tm_hour -= tm->tm_hour % grid_align_hours; // align to grid window
       kodi::Log(ADDON_LOG_DEBUG, "[epg-new] tm %d", tm->tm_hour);
 
-      char startTimeBuf[30];
+
       // 2024-05-17T17:00:00.000Z
       strftime(startTimeBuf, 30, "%Y-%m-%dT%H:00:00.000Z", tm);
 
@@ -1323,8 +1153,8 @@ PVR_ERROR WaipuData::GetEPGForChannelNew(int channelUid,
       }
       jsonEpg = "{\"result\": " + jsonEpg + "}";
 
-      rapidjson::Document epgDoc;
-      epgDoc.Parse(jsonEpg.c_str());
+
+      epgDoc.ParseInsitu<rapidjson::kParseInsituFlag>(&jsonEpg[0]);
       if (epgDoc.HasParseError())
       {
         kodi::Log(ADDON_LOG_ERROR, "[GetEPG] ERROR: error while parsing json");
@@ -1337,11 +1167,11 @@ PVR_ERROR WaipuData::GetEPGForChannelNew(int channelUid,
       {
         kodi::addon::PVREPGTag tag;
 
-        // generate a unique boadcast id
-        const std::string epg_bid = epgData["id"].GetString();
-        kodi::Log(ADDON_LOG_DEBUG, "[epg] epg_bid: %s;", epg_bid.c_str());
-        int dirtyID = Utils::GetIDDirty(epg_bid);
-        kodi::Log(ADDON_LOG_DEBUG, "[epg] epg_bid dirty: %i;", dirtyID);
+        // get id, example: 1f2b25b5-0a39-5957-b411-3825326a3e54
+        const std::string epg_id = epgData["id"].GetString();
+        kodi::Log(ADDON_LOG_DEBUG, "[epg] epg_id: %s;", epg_id.c_str());
+        int dirtyID = Utils::GetIDDirty(epg_id);
+        kodi::Log(ADDON_LOG_DEBUG, "[epg] epg_id dirty: %i;", dirtyID);
         tag.SetUniqueBroadcastId(dirtyID);
 
         // channel ID
@@ -1370,6 +1200,10 @@ PVR_ERROR WaipuData::GetEPGForChannelNew(int channelUid,
         const std::string entryEndTime = epgData["stopTime"].GetString();
         tag.SetEndTime(Utils::StringToTime(entryEndTime));
 
+        // set SetSeriesLink
+        // example: https://play.waipu.tv/programm/n24/1f2b25b5-0a39-5957-b411-3825326a3e54
+        tag.SetSeriesLink("https://play.waipu.tv/programm/"+channelid+"/"+epg_id);
+
         // epg preview image
         if (m_epg_show_preview_images && epgData.HasMember("previewImage"))
         {
@@ -1845,11 +1679,12 @@ PVR_ERROR WaipuData::DeleteRecording(const kodi::addon::PVRRecording& recording)
     return PVR_ERROR_FAILED;
 
   std::string recording_id = recording.GetRecordingId();
+  // {"ids":["1310978347"]}
   std::string request_data = "{\"ids\":[\"" + recording_id + "\"]}";
   kodi::Log(ADDON_LOG_DEBUG, "[delete recording] req: %s;", request_data.c_str());
   std::string deleted =
       HttpDelete("https://recording.waipu.tv/api/recordings", request_data,
-                 {{"Content-Type", "application/vnd.waipu.pvr-recording-ids-v2+json"}});
+                 {{"Content-Type", "application/vnd.waipu.recording-ids-v4+json"}});
   kodi::Log(ADDON_LOG_DEBUG, "[delete recording] response: %s;", deleted.c_str());
   kodi::addon::CInstancePVRClient::TriggerRecordingUpdate();
   return PVR_ERROR_NO_ERROR;
@@ -2104,41 +1939,25 @@ PVR_ERROR WaipuData::AddTimer(const kodi::addon::PVRTimer& timer)
   if (!IsConnected())
     return PVR_ERROR_FAILED;
 
-  for (const auto& channel : m_channels)
-  {
-    if (channel.iUniqueId != timer.GetClientChannelUid())
-      continue;
+  // Get UUID from SeriesLink - workaround: better save UUID somewhere else
+  const std::string series_link = timer.GetSeriesLink().c_str();
+  kodi::Log(ADDON_LOG_DEBUG, "[add timer] Series Link: %s", series_link.c_str());
+  std::vector<std::string> urlsplit;
+  urlsplit = kodi::tools::StringUtils::Split(series_link, "/");
+  const std::string programId = urlsplit.back();
+  kodi::Log(ADDON_LOG_DEBUG, "[add timer] ProgramId: %s", programId.c_str());
+
+  // record single element
+  kodi::Log(ADDON_LOG_DEBUG, "[add timer] Record single tag;");
+  // {"programId":"_1051966761","channelId":"PRO7","startTime":"2019-02-03T18:05:00.000Z","stopTime":"2019-02-03T19:15:00.000Z"}
+  std::string postData = "{\"programId\":\"" + programId + "\"}";
+  std::string recordResp =
+      HttpPost("https://recording.waipu.tv/api/recordings", postData,
+               {{"Content-Type", "application/vnd.waipu.recording-create-v4+json"}});
+  kodi::Log(ADDON_LOG_DEBUG, "[add timer] single response: %s;", recordResp.c_str());
+  kodi::QueueNotification(QUEUE_INFO, "Recording", "Recording Created");
+  kodi::addon::CInstancePVRClient::TriggerTimerUpdate();
 
-    if (timer.GetTimerType() == 1)
-    {
-      // record single element
-      kodi::Log(ADDON_LOG_DEBUG, "[add timer] Record single tag;");
-      // {"programId":"_1051966761","channelId":"PRO7","startTime":"2019-02-03T18:05:00.000Z","stopTime":"2019-02-03T19:15:00.000Z"}
-      std::string postData = "{\"programId\":\"_" + std::to_string(timer.GetEPGUid()) +
-                             "\",\"channelId\":\"" + channel.waipuID + "\"" + "}";
-      std::string recordResp =
-          HttpPost("https://recording.waipu.tv/api/recordings", postData,
-                   {{"Content-Type", "application/vnd.waipu.start-recording-v2+json"}});
-      kodi::Log(ADDON_LOG_DEBUG, "[add timer] single response: %s;", recordResp.c_str());
-      kodi::QueueNotification(QUEUE_INFO, "Recording", "Recording Created");
-      kodi::addon::CInstancePVRClient::TriggerTimerUpdate();
-    }
-    else
-    {
-      // record series
-      kodi::Log(ADDON_LOG_DEBUG, "[add timer] Record single tag;");
-      // {"title":"Das A-Team","channel":"RTLNITRO"}
-      std::string postData =
-          "{\"title\": \"" + timer.GetTitle() + "\",\"channel\":\"" + channel.waipuID + "\"" + "}";
-      std::string recordResp =
-          HttpPost("https://recording-scheduler.waipu.tv/api/serials", postData,
-                   {{"Content-Type", "application/vnd.waipu.recording-scheduler-serials-v1+json"}});
-      kodi::Log(ADDON_LOG_DEBUG, "[add timer] repeating response: %s;", recordResp.c_str());
-      kodi::QueueNotification(QUEUE_INFO, "Recording", "Rule Created");
-      kodi::addon::CInstancePVRClient::TriggerRecordingUpdate();
-      kodi::addon::CInstancePVRClient::TriggerTimerUpdate();
-    }
-  }
   return PVR_ERROR_NO_ERROR;
 }
 
